@@ -1,3 +1,7 @@
+# Licensed to Elasticsearch B.V under one or more agreements.
+# Elasticsearch B.V licenses this file to you under the Apache 2.0 License.
+# See the LICENSE file in the project root for more information
+
 from .utils import NamespacedClient, query_params, _make_path, SKIP_IN_PATH
 
 
@@ -6,7 +10,7 @@ class CcrClient(NamespacedClient):
     def delete_auto_follow_pattern(self, name, params=None, headers=None):
         """
         Deletes auto-follow patterns.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-delete-auto-follow-pattern.html>`_
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.8/ccr-delete-auto-follow-pattern.html>`_
 
         :arg name: The name of the auto follow pattern.
         """
@@ -24,7 +28,7 @@ class CcrClient(NamespacedClient):
     def follow(self, index, body, params=None, headers=None):
         """
         Creates a new follower index configured to follow the referenced leader index.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-put-follow.html>`_
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.8/ccr-put-follow.html>`_
 
         :arg index: The name of the follower index
         :arg body: The name of the leader index and other optional ccr
@@ -33,7 +37,7 @@ class CcrClient(NamespacedClient):
             that must be active before returning. Defaults to 0. Set to `all` for
             all shard copies, otherwise set to any non-negative value less than or
             equal to the total number of copies for the shard (number of replicas +
-            1)    Default: 0
+            1)  Default: 0
         """
         for param in (index, body):
             if param in SKIP_IN_PATH:
@@ -52,7 +56,7 @@ class CcrClient(NamespacedClient):
         """
         Retrieves information about all follower indices, including parameters and
         status for each follower index
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-get-follow-info.html>`_
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.8/ccr-get-follow-info.html>`_
 
         :arg index: A comma-separated list of index patterns; use `_all`
             to perform the operation on all indices
@@ -69,7 +73,7 @@ class CcrClient(NamespacedClient):
         """
         Retrieves follower stats. return shard-level stats about the following tasks
         associated with each shard for the specified indices.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-get-follow-stats.html>`_
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.8/ccr-get-follow-stats.html>`_
 
         :arg index: A comma-separated list of index patterns; use `_all`
             to perform the operation on all indices
@@ -85,7 +89,7 @@ class CcrClient(NamespacedClient):
     def forget_follower(self, index, body, params=None, headers=None):
         """
         Removes the follower retention leases from the leader.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-post-forget-follower.html>`_
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.8/ccr-post-forget-follower.html>`_
 
         :arg index: the name of the leader index for which specified
             follower retention leases should be removed
@@ -111,7 +115,7 @@ class CcrClient(NamespacedClient):
         """
         Gets configured auto-follow patterns. Returns the specified auto-follow pattern
         collection.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-get-auto-follow-pattern.html>`_
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.8/ccr-get-auto-follow-pattern.html>`_
 
         :arg name: The name of the auto follow pattern.
         """
@@ -127,7 +131,7 @@ class CcrClient(NamespacedClient):
         """
         Pauses a follower index. The follower index will not fetch any additional
         operations from the leader index.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-post-pause-follow.html>`_
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.8/ccr-post-pause-follow.html>`_
 
         :arg index: The name of the follower index that should pause
             following its leader index.
@@ -148,7 +152,7 @@ class CcrClient(NamespacedClient):
         Creates a new named collection of auto-follow patterns against a specified
         remote cluster. Newly created indices on the remote cluster matching any of the
         specified patterns will be automatically configured as follower indices.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-put-auto-follow-pattern.html>`_
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.8/ccr-put-auto-follow-pattern.html>`_
 
         :arg name: The name of the auto follow pattern.
         :arg body: The specification of the auto follow pattern
@@ -169,7 +173,7 @@ class CcrClient(NamespacedClient):
     def resume_follow(self, index, body=None, params=None, headers=None):
         """
         Resumes a follower index that has been paused
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-post-resume-follow.html>`_
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.8/ccr-post-resume-follow.html>`_
 
         :arg index: The name of the follow index to resume following.
         :arg body: The name of the leader index and other optional ccr
@@ -190,7 +194,7 @@ class CcrClient(NamespacedClient):
     def stats(self, params=None, headers=None):
         """
         Gets all stats related to cross-cluster replication.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-get-stats.html>`_
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.8/ccr-get-stats.html>`_
         """
         return self.transport.perform_request(
             "GET", "/_ccr/stats", params=params, headers=headers
@@ -201,7 +205,7 @@ class CcrClient(NamespacedClient):
         """
         Stops the following task associated with a follower index and removes index
         metadata and settings associated with cross-cluster replication.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-post-unfollow.html>`_
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.8/ccr-post-unfollow.html>`_
 
         :arg index: The name of the follower index that should be turned
             into a regular index.
@@ -220,7 +224,7 @@ class CcrClient(NamespacedClient):
     def pause_auto_follow_pattern(self, name, params=None, headers=None):
         """
         Pauses an auto-follow pattern
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-pause-auto-follow-pattern.html>`_
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.8/ccr-pause-auto-follow-pattern.html>`_
 
         :arg name: The name of the auto follow pattern that should pause
             discovering new indices to follow.
@@ -239,7 +243,7 @@ class CcrClient(NamespacedClient):
     def resume_auto_follow_pattern(self, name, params=None, headers=None):
         """
         Resumes an auto-follow pattern that has been paused
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-resume-auto-follow-pattern.html>`_
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.8/ccr-resume-auto-follow-pattern.html>`_
 
         :arg name: The name of the auto follow pattern to resume
             discovering new indices to follow.
