@@ -1,3 +1,7 @@
+# Licensed to Elasticsearch B.V under one or more agreements.
+# Elasticsearch B.V licenses this file to you under the Apache 2.0 License.
+# See the LICENSE file in the project root for more information
+
 from .utils import NamespacedClient, query_params, _make_path, SKIP_IN_PATH
 
 
@@ -7,7 +11,7 @@ class GraphClient(NamespacedClient):
         """
         Explore extracted and summarized information about the documents and terms in
         an index.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/graph-explore-api.html>`_
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.8/graph-explore-api.html>`_
 
         :arg index: A comma-separated list of index names to search; use
             `_all` or empty string to perform the operation on all indices
