@@ -1,3 +1,7 @@
+# Licensed to Elasticsearch B.V under one or more agreements.
+# Elasticsearch B.V licenses this file to you under the Apache 2.0 License.
+# See the LICENSE file in the project root for more information
+
 from operator import methodcaller
 import time
 
@@ -27,22 +31,32 @@ def expand_action(data):
     op_type = data.pop("_op_type", "index")
     action = {op_type: {}}
     for key in (
+        "_id",
         "_index",
         "_parent",
         "_percolate",
+        "_retry_on_conflict",
         "_routing",
         "_timestamp",
-        "routing",
         "_type",
         "_version",
         "_version_type",
-        "_id",
-        "retry_on_conflict",
+        "parent",
         "pipeline",
+        "retry_on_conflict",
+        "routing",
+        "version",
+        "version_type",
     ):
         if key in data:
-            if key == "_routing":
-                action[op_type]["routing"] = data.pop(key)
+            if key in [
+                "_parent",
+                "_retry_on_conflict",
+                "_routing",
+                "_version",
+                "_version_type",
+            ]:
+                action[op_type][key[1:]] = data.pop(key)
             else:
                 action[op_type][key] = data.pop(key)
 
@@ -53,91 +67,77 @@ def expand_action(data):
     return action, data.get("_source", data)
 
 
+class _ActionChunker:
+    def __init__(self, chunk_size, max_chunk_bytes, serializer):
+        self.chunk_size = chunk_size
+        self.max_chunk_bytes = max_chunk_bytes
+        self.serializer = serializer
+
+        self.size = 0
+        self.action_count = 0
+        self.bulk_actions = []
+        self.bulk_data = []
+
+    def feed(self, action, data):
+        ret = None
+        raw_data, raw_action = data, action
+        action = self.serializer.dumps(action)
+        # +1 to account for the trailing new line character
+        cur_size = len(action.encode("utf-8")) + 1
+
+        if data is not None:
+            data = self.serializer.dumps(data)
+            cur_size += len(data.encode("utf-8")) + 1
+
+        # full chunk, send it and start a new one
+        if self.bulk_actions and (
+            self.size + cur_size > self.max_chunk_bytes
+            or self.action_count == self.chunk_size
+        ):
+            ret = (self.bulk_data, self.bulk_actions)
+            self.bulk_actions, self.bulk_data = [], []
+            self.size, self.action_count = 0, 0
+
+        self.bulk_actions.append(action)
+        if data is not None:
+            self.bulk_actions.append(data)
+            self.bulk_data.append((raw_action, raw_data))
+        else:
+            self.bulk_data.append((raw_action,))
+
+        self.size += cur_size
+        self.action_count += 1
+        return ret
+
+    def flush(self):
+        ret = None
+        if self.bulk_actions:
+            ret = (self.bulk_data, self.bulk_actions)
+            self.bulk_actions, self.bulk_data = [], []
+        return ret
+
+
 def _chunk_actions(actions, chunk_size, max_chunk_bytes, serializer):
     """
     Split actions into chunks by number or size, serialize them into strings in
     the process.
     """
-    bulk_actions, bulk_data = [], []
-    size, action_count = 0, 0
+    chunker = _ActionChunker(
+        chunk_size=chunk_size, max_chunk_bytes=max_chunk_bytes, serializer=serializer
+    )
     for action, data in actions:
-        raw_data, raw_action = data, action
-        action = serializer.dumps(action)
-        # +1 to account for the trailing new line character
-        cur_size = len(action.encode("utf-8")) + 1
-
-        if data is not None:
-            data = serializer.dumps(data)
-            cur_size += len(data.encode("utf-8")) + 1
-
-        # full chunk, send it and start a new one
-        if bulk_actions and (
-            size + cur_size > max_chunk_bytes or action_count == chunk_size
-        ):
-            yield bulk_data, bulk_actions
-            bulk_actions, bulk_data = [], []
-            size, action_count = 0, 0
-
-        bulk_actions.append(action)
-        if data is not None:
-            bulk_actions.append(data)
-            bulk_data.append((raw_action, raw_data))
-        else:
-            bulk_data.append((raw_action,))
-
-        size += cur_size
-        action_count += 1
-
-    if bulk_actions:
-        yield bulk_data, bulk_actions
+        ret = chunker.feed(action, data)
+        if ret:
+            yield ret
+    ret = chunker.flush()
+    if ret:
+        yield ret
 
 
-def _process_bulk_chunk(
-    client,
-    bulk_actions,
-    bulk_data,
-    raise_on_exception=True,
-    raise_on_error=True,
-    *args,
-    **kwargs
-):
-    """
-    Send a bulk request to elasticsearch and process the output.
-    """
+def _process_bulk_chunk_success(resp, bulk_data, raise_on_error=True):
     # if raise on error is set, we need to collect errors per chunk before raising them
     errors = []
 
-    try:
-        # send the actual request
-        resp = client.bulk("\n".join(bulk_actions) + "\n", *args, **kwargs)
-    except TransportError as e:
-        # default behavior - just propagate exception
-        if raise_on_exception:
-            raise e
-
-        # if we are not propagating, mark all actions in current chunk as failed
-        err_message = str(e)
-        exc_errors = []
-
-        for data in bulk_data:
-            # collect all the information about failed actions
-            op_type, action = data[0].copy().popitem()
-            info = {"error": err_message, "status": e.status_code, "exception": e}
-            if op_type != "delete":
-                info["data"] = data[1]
-            info.update(action)
-            exc_errors.append({op_type: info})
-
-        # emulate standard behavior for failed actions
-        if raise_on_error:
-            raise BulkIndexError(
-                "%i document(s) failed to index." % len(exc_errors), exc_errors
-            )
-        else:
-            for err in exc_errors:
-                yield False, err
-            return
-
     # go through request-response pairs and detect failures
     for data, (op_type, item) in zip(
         bulk_data, map(methodcaller("popitem"), resp["items"])
@@ -158,6 +158,66 @@ def _process_bulk_chunk(
         raise BulkIndexError("%i document(s) failed to index." % len(errors), errors)
 
 
+def _process_bulk_chunk_error(
+    error, bulk_data, raise_on_exception=True, raise_on_error=True
+):
+    # default behavior - just propagate exception
+    if raise_on_exception:
+        raise error
+
+    # if we are not propagating, mark all actions in current chunk as failed
+    err_message = str(error)
+    exc_errors = []
+
+    for data in bulk_data:
+        # collect all the information about failed actions
+        op_type, action = data[0].copy().popitem()
+        info = {"error": err_message, "status": error.status_code, "exception": error}
+        if op_type != "delete":
+            info["data"] = data[1]
+        info.update(action)
+        exc_errors.append({op_type: info})
+
+    # emulate standard behavior for failed actions
+    if raise_on_error:
+        raise BulkIndexError(
+            "%i document(s) failed to index." % len(exc_errors), exc_errors
+        )
+    else:
+        for err in exc_errors:
+            yield False, err
+
+
+def _process_bulk_chunk(
+    client,
+    bulk_actions,
+    bulk_data,
+    raise_on_exception=True,
+    raise_on_error=True,
+    *args,
+    **kwargs
+):
+    """
+    Send a bulk request to elasticsearch and process the output.
+    """
+    try:
+        # send the actual request
+        resp = client.bulk("\n".join(bulk_actions) + "\n", *args, **kwargs)
+    except TransportError as e:
+        gen = _process_bulk_chunk_error(
+            error=e,
+            bulk_data=bulk_data,
+            raise_on_exception=raise_on_exception,
+            raise_on_error=raise_on_error,
+        )
+    else:
+        gen = _process_bulk_chunk_success(
+            resp=resp, bulk_data=bulk_data, raise_on_error=raise_on_error
+        )
+    for item in gen:
+        yield item
+
+
 def streaming_bulk(
     client,
     actions,
@@ -347,7 +407,9 @@ def parallel_bulk(
     class BlockingPool(ThreadPool):
         def _setup_queues(self):
             super(BlockingPool, self)._setup_queues()
-            self._inqueue = Queue(queue_size)
+            # The queue must be at least the size of the number of threads to
+            # prevent hanging when inserting sentinel values during teardown.
+            self._inqueue = Queue(max(queue_size, thread_count))
             self._quick_put = self._inqueue.put
 
     pool = BlockingPool(thread_count)
@@ -435,29 +497,32 @@ def scan(
     scroll_id = resp.get("_scroll_id")
 
     try:
-        while scroll_id and resp['hits']['hits']:
+        while scroll_id and resp["hits"]["hits"]:
             for hit in resp["hits"]["hits"]:
                 yield hit
 
             # check if we have any errors
-            if resp["_shards"]["successful"] < resp["_shards"]["total"]:
+            if (resp["_shards"]["successful"] + resp["_shards"]["skipped"]) < resp[
+                "_shards"
+            ]["total"]:
                 logger.warning(
-                    "Scroll request has only succeeded on %d shards out of %d.",
+                    "Scroll request has only succeeded on %d (+%d skipped) shards out of %d.",
                     resp["_shards"]["successful"],
+                    resp["_shards"]["skipped"],
                     resp["_shards"]["total"],
                 )
                 if raise_on_error:
                     raise ScanError(
                         scroll_id,
-                        "Scroll request has only succeeded on %d shards out of %d."
-                        % (resp["_shards"]["successful"], resp["_shards"]["total"]),
+                        "Scroll request has only succeeded on %d (+%d skiped) shards out of %d."
+                        % (
+                            resp["_shards"]["successful"],
+                            resp["_shards"]["skipped"],
+                            resp["_shards"]["total"],
+                        ),
                     )
-
             resp = client.scroll(
-                scroll_id,
-                scroll=scroll,
-                request_timeout=request_timeout,
-                **scroll_kwargs
+                body={"scroll_id": scroll_id, "scroll": scroll}, **scroll_kwargs
             )
             scroll_id = resp.get("_scroll_id")
 
